type QueryStr = str
type QueryInt = int
type QueryFloat = float
type QueryBool = bool
type PathStr = str
type PathInt = int
type PathFloat = float
type PathBool = bool
type HeaderStr = str
type HeaderInt = int
type HeaderFloat = float
type HeaderBool = bool
type CookieStr = str
type CookieInt = int
type CookieFloat = float
type CookieBool = bool
type BodyDict = dict
